#!/usr/bin/python

import sys,re,os;
import tempfile;
import shutil;
import datetime;

strBasedir = '/server/topcons_single';

strMemsat         = '/server/bin/memsat';
strMemsatdir      = os.path.dirname(strMemsat);
strScampidir      = '/server/scampi';
strModhmmdir      = '/server/topcons_single';

strSpoctopusdir   = '/server/spoctopus';
strModhmms        = '/data3/server/modhmm/bin/modhmms';
strModhmms_scampi = '/data3/server/modhmm/bin/modhmms_scampi';
strStmhmmModel    = '/server/prodiv_tmhmm/HMM_FILES/S_TMHMM_0.92b.txt';
strProdivtmhmmModel    = '/server/prodiv_tmhmm/HMM_FILES/PRODIV_TMHMM_0.92b.txt';
strPhobiusModel    = os.path.join(strBasedir, 'PHOBIUS.txt');
strModhmmRepl     = '/server/prodiv_tmhmm/util/replacement_letter_multi.rpl';
strModhmmPri      = '/server/prodiv_tmhmm/util/amino_multi.pri';

strHmmtopdir      = '/server/topcons_single';
strToppred        = '/server/bin/toppred';
strToppreddir     = os.path.dirname(strToppred).rstrip('bin'); # ugly fix
strToppredDatadir = os.path.join(strToppreddir, 'share','toppred');
# strTmhmm2dir      = '/afs/pdc.kth.se/home/a/aronh/1vol/TMHMM2.0';
# strTmhmm2         = os.path.join(strTmhmm2dir, 'bin/tmhmm');
strFakeProfiledir = '/server/topcons_single';
# strOctopus        = '/server/spoctopus/OCTOPUS.sh';
# strCsblastdir   = '/afs/pdc.kth.se/home/a/aronh/4vol/topology-sandbox/csblast';
# strCsprofilebin = '/afs/pdc.kth.se/home/a/aronh/4vol/topology-sandbox/csblast/csprofile.py';
# strMtx2prf      = '/server/spoctopus/modhmmblast/mtx2prf.sh';
modhmm2top = '/data3/server/modhmm/bin/modhmmxml2top';
fasta_split = '/server/topcons_single/fasta_split.py';


def run_memsat(lstSeqfiles, strOutdir, debug = False) :
    tmpdir = tempfile.mkdtemp(prefix='memsat_tmp_');
    sys.path.append(strBasedir);
    import memsat;
    for strFastaFile in lstSeqfiles :
        flh = file(strFastaFile);
        foo = flh.readline();
        seq = flh.read().replace('\n','').strip();
        flh.close();
        if len(seq) > 2000 :
            flhErr = file(os.path.join(strOutdir, strChainName + '.error'), 'w');
            print >>flhErr, "Sequence too long for memsat (>2000): '%d'" % len(seq);
            flhErr.close();
            return ;
        strChainName = os.path.basename(strFastaFile)[:-3];
        strResFile = os.path.join(tmpdir, strChainName + '.memsat.res');
        strTopFile = os.path.join(strOutdir, strChainName + '.top');
        os.system('cd %s; %s %s > %s' % (strMemsatdir, strMemsat, strFastaFile, strResFile));
        try :
            memsat.memsat1res2top(strResFile, strTopFile);
        except Exception, e:
            flhErr = file(os.path.join(strOutdir, strChainName + '.error'), 'w');
            print >>flhErr, e;
            flhErr.close();
    if debug :
        shutil.copytree(tmpdir, os.path.join(strOutdir, 'tmpdir'));
    shutil.rmtree(tmpdir);
    

def run_scampi_seq(lstSeqfiles, strOutdir, debug  = False) :
    tmpdir = tempfile.mkdtemp(prefix='scampi_tmp_');
    strNamesfile = os.path.join(tmpdir, 'names.txt');
    flhNames = file(strNamesfile, 'w');
    for strFastaFile in lstSeqfiles :
        print >>flhNames, strFastaFile.strip();
    flhNames.close();
    strResfile = os.path.join(tmpdir, 'outfile.top');
    lllabeling = '';
    nolabels = '--nolabels';
    if debug :
        lllabeling = '--labelllout --labeloddsout --labelrevout'; # print various scores to xml output
        nolabels = ''; # use labels if present in input file
    dctArgs = {'namefile': strNamesfile,
               'scampidir': strScampidir,
               'tmpdir': tmpdir,
               'lllabeling': lllabeling,
               'nolabels': nolabels};
    # With -g flag (globular protein filter)
    os.system("/data3/server/modhmm/bin/modhmms_scampi -f fa -s %(namefile)s -m %(scampidir)s/DGHMM_KR_21.txt -o %(tmpdir)s -r %(scampidir)s/replacement_letter_multi.rpl --nopostout %(nolabels)s %(lllabeling)s --viterbi -u -L -g > %(tmpdir)s/outfile.xml" % dctArgs);
    os.system("/bin/cat %(tmpdir)s/outfile.xml | /data3/server/modhmm/bin/modhmmxml2top > %(outfile)s" % {'tmpdir': tmpdir, 'outfile': strResfile});
    os.system('%s %s %s top 0 " "' % (fasta_split, strResfile, strOutdir));
    if debug :
        shutil.copytree(tmpdir, os.path.join(strOutdir, 'tmpdir'));
    shutil.rmtree(tmpdir);

def run_phobius_seq(lstSeqfiles, strOutdir, debug = False) :
    tmpdir = tempfile.mkdtemp(prefix='phobius_tmp_');
    strNamesfile = os.path.join(tmpdir, 'names.txt');
    flhNames = file(strNamesfile, 'w');
    for strFastaFile in lstSeqfiles :
        print >>flhNames, strFastaFile.strip();
    flhNames.close();
    strResfile = os.path.join(tmpdir, 'outfile.top');
    lllabeling = '';
    nolabels = '--nolabels';
    if debug :
        lllabeling = '--labelllout --labeloddsout --labelrevout'; # print various scores to xml output
        nolabels = ''; # use labels if present in input file
    dctArgs = {'namefile': strNamesfile,
               'model': strPhobiusModel,
               'repl': strModhmmRepl,
               'tmpdir': tmpdir,
               'lllabeling': lllabeling,
               'nolabels': nolabels};
    os.system("/data3/server/modhmm/bin/modhmms -f fa -s %(namefile)s -m %(model)s -o %(tmpdir)s -r %(repl)s --nopostout %(nolabels)s %(lllabeling)s -L > %(tmpdir)s/outfile.xml" % dctArgs);
    os.system("/bin/cat %(tmpdir)s/outfile.xml | /data3/server/modhmm/bin/modhmmxml2top > %(outfile)s" % {'tmpdir': tmpdir, 'outfile': strResfile});
    os.system('%s %s %s top 0 " "' % (fasta_split, strResfile, strOutdir));
    if debug :
        shutil.copytree(tmpdir, os.path.join(strOutdir, 'tmpdir'));
    shutil.rmtree(tmpdir);


def run_stmhmm(lstSeqfiles, strOutdir, debug = False) :
    tmpdir = tempfile.mkdtemp(prefix='stmhmm_tmp_');
    strNamesfile = os.path.join(tmpdir, 'names.txt');
    flhNames = file(strNamesfile, 'w');
    for strFastaFile in lstSeqfiles :
        print >>flhNames, strFastaFile.strip();
    flhNames.close();
    strResfile = os.path.join(tmpdir, 'outfile.top');
    lllabeling = '';
    nolabels = '--nolabels';
    if debug :
        lllabeling = '--labelllout --labeloddsout --labelrevout'; # print various scores to xml output
        nolabels = ''; # use labels if present in input file
    dctArgs = {'namefile': strNamesfile,
               'model': strStmhmmModel,
               'repl': strModhmmRepl,
               'tmpdir': tmpdir,
               'lllabeling': lllabeling,
               'nolabels': nolabels};
    os.system("/data3/server/modhmm/bin/modhmms -f fa -s %(namefile)s -m %(model)s -o %(tmpdir)s -r %(repl)s --nopostout %(nolabels)s %(lllabeling)s -L > %(tmpdir)s/outfile.xml" % dctArgs);
    os.system("/bin/cat %(tmpdir)s/outfile.xml | /data3/server/modhmm/bin/modhmmxml2top > %(outfile)s" % {'tmpdir': tmpdir, 'outfile': strResfile});
    os.system('%s %s %s top 0 " "' % (fasta_split, strResfile, strOutdir));
    if debug :
        shutil.copytree(tmpdir, os.path.join(strOutdir, 'tmpdir'));
    shutil.rmtree(tmpdir);

def run_sdivtmhmm(lstSeqfiles, strOutdir, debug = False) :
    tmpdir = tempfile.mkdtemp(prefix='sdivtmhmm_tmp_');
    rawprfdir = os.path.join(tmpdir, 'rawprf');
    os.mkdir(rawprfdir);
    pssmdir = os.path.join(tmpdir, 'pssm');
    os.mkdir(pssmdir);
    
    import fake_profile;
    for seqfile in lstSeqfiles :
        fake_profile.createProfile(os.path.basename(seqfile)[:-3], os.path.dirname(seqfile), rawprfdir, 'blast');

    strPrfnames = os.path.join(tmpdir, 'prfs');
    flhPrfs = file(strPrfnames, 'w');
    for seq in lstSeqfiles :
        print >>flhPrfs, os.path.join(rawprfdir, os.path.basename(seq)[:-3] + '.prf');
    flhPrfs.close();

    sys.path.append(strModhmmdir);
    import modhmm;
    class sdivtmhmm_execModhmm(modhmm.modhmm.execModhmm) :
        def runSdivtmhmm(self) :
            names = strPrfnames;
            res = self._run(self.prodiv_model, 'prf', names);
            os.remove(names);
            return res;
    m = modhmm.modhmm();
    prdctr = sdivtmhmm_execModhmm(lstSeqfiles, os.path.dirname(lstSeqfiles[0]), tmpdir, tmpdir, tmpdir, False);
    prdctr.modhmms = strModhmms;
    prdctr.prodiv_model = strStmhmmModel;
    prdctr.repl = strModhmmRepl;
    prdctr.pri = strModhmmPri;
    output = prdctr.runSdivtmhmm();
    strResXML = os.path.join(tmpdir, 'modhmms_res.xml');
    flhOut = file(strResXML, 'w');
    print >>flhOut, output;
    flhOut.close();
    strResfile = os.path.join(tmpdir, 'modhmms_res.top');
    os.system("/bin/cat %s | %s > %s" % (strResXML, modhmm2top, strResfile));
    os.system('%s %s %s top 0 " "' % (fasta_split, strResfile, strOutdir));
    if debug :
        shutil.copytree(tmpdir, os.path.join(strOutdir, 'tmpdir'));
    shutil.rmtree(tmpdir);

def run_octopus_seq(lstSeqfiles, strOutdir, debug = False) :
    tmpdir = tempfile.mkdtemp(prefix='octopus_seq_tmp_');
    rawprfdir = os.path.join(tmpdir, 'rawprf');
    os.mkdir(rawprfdir);
    pssmdir = os.path.join(tmpdir, 'pssm');
    os.mkdir(pssmdir);

    strChainNames = os.path.join(tmpdir, 'chainnames');
    flhChainNames = file(strChainNames, 'w');
    for seq in lstSeqfiles :
        print >>flhChainNames, os.path.basename(seq)[:-3];
    flhChainNames.close();
    
    import fake_profile;
    for seqfile in lstSeqfiles :
        fake_profile.createProfile(os.path.basename(seqfile)[:-3], os.path.dirname(seqfile), pssmdir, 'psiblast');
        fake_profile.createProfile(os.path.basename(seqfile)[:-3], os.path.dirname(seqfile), rawprfdir, 'blast');
    
    os.system("%s %s %s %s %s" % (strOctopus, strChainNames, pssmdir, rawprfdir, strOutdir));
    
    if debug :
        shutil.copytree(tmpdir, os.path.join(strOutdir, 'tmpdir'));
    shutil.rmtree(tmpdir);


def run_hmmtop(lstSeqfiles, strOutdir, debug = False) :
    tmpdir = tempfile.mkdtemp(prefix='hmmtop_tmp_');
    sys.path.append(strHmmtopdir);
    import hmmtop;
    h = hmmtop.hmmtop();
    for strFastaFile in lstSeqfiles :
        strChainName = os.path.basename(strFastaFile)[:-3];
        strHmmtopresFile = os.path.join(tmpdir, strChainName + '.hmmtopres');
        strHmmtoptopFile = os.path.join(strOutdir, strChainName + '.top');
        h.run(strFastaFile,strHmmtopresFile, print_probabilities = debug);
        try :
            h.res2top(strHmmtopresFile, strHmmtoptopFile, strFastaFile);
        except Exception, e:
            flhErr = file(os.path.join(strOutdir, strChainName + '.error'), 'w');
            print >>flhErr, e;
            shutil.copy(strHmmtopresFile, os.path.join(strOutdir, strChainName + '.hmmtopres'));
            flhErr.close();    
    if debug :
        shutil.copytree(tmpdir, os.path.join(strOutdir, 'tmpdir'));
    shutil.rmtree(tmpdir);

def run_toppred(lstSeqfiles, strOutdir, debug = False) :
    tmpdir = tempfile.mkdtemp(prefix='toppred_tmp__');
    sys.path.append(strToppreddir);
    import toppred;
    for strFastaFile in lstSeqfiles :
        strChainName = os.path.basename(strFastaFile)[:-3];
        strResFile = os.path.join(tmpdir, strChainName + '.toppredres');
        strTopFile = os.path.join(strOutdir, strChainName + '.top');
        os.system('export TOPPREDDATA=%s; cd %s; %s -g none %s > %s' % (strToppredDatadir, tmpdir, strToppred, strFastaFile, strResFile));
        lstXs = toppred.findFastaX(strFastaFile);
        try :
            toppred.toppredres2top(strResFile, strTopFile, lstXs);
        except Exception, e:
            flhErr = file(os.path.join(strOutdir, strChainName + '.error'), 'w');
            print >>flhErr, e;
            flhErr.close();
            shutil.copy(strResFile, os.path.join(strOutdir, strChainName + '.toppredres'));
        lstGarbagefiles = [os.path.join(os.environ['PWD'], strChainName + '-1.png'),\
                               os.path.join(os.environ['PWD'], strChainName + '-2.png'),\
                               os.path.join(os.environ['PWD'], strChainName + '.hydro')];
        for strFile in lstGarbagefiles :
            if os.path.isfile(strFile) : os.remove(strFile);
    if debug :
        shutil.copytree(tmpdir, os.path.join(strOutdir, 'tmpdir'));
    shutil.rmtree(tmpdir);
    

# def run_tmhmm2(lstSeqfiles, strOutdir) :
#     tmpdir = tempfile.mkdtemp(prefix='tmhmm2.0_tmp_');
#     sys.path.append(strTmhmm2dir);
#     import tmhmm2;
#     for strFastaFile in lstSeqfiles :
#         strChainName = os.path.basename(strFastaFile)[:-3];
#         strResFile = os.path.join(tmpdir, strChainName + '.fa.res');
#         strTopFile = os.path.join(strOutdir, strChainName + '.top');
#         try :
#             os.system('cat %s | %s > %s' % (strFastaFile, strTmhmm2, strResFile));
#         except Exception, e:
#             flhErr = file(os.path.join(strOutdir, strChainName + '.error'), 'w');
#             print >>flhErr, e;
#             flhErr.close();
#         tmhmm2.tmhmm2res2top(strResFile, strTopFile);
#     lstGarbagedirs = filter(lambda x:os.path.isdir(x) and re.match('^\d+$', x), os.listdir(os.environ['PWD']));
#     for strGarbagedir in lstGarbagedirs :
#         shutil.rmtree(strGarbagedir);
#     shutil.rmtree(tmpdir);

# def run_phillius() :
#     pass ;

def run_octopus_csblast(lstSeqfiles, strOutdir, debug = False) :
    tmpdir = tempfile.mkdtemp(prefix='octopus_csblast_tmp_');
    sys.path.append(strCsblastdir);
    from csprofile_pipeline import csprofile;
    c = csprofile();
    c.strCsprofile_script = strCsprofilebin;
    # c.strMakematdir = '/path/to/makemat'; # fix

    for strFastaFile in lstSeqfiles :
        strChainName = os.path.splitext(os.path.basename(strFastaFile))[0];
        strResFile = os.path.join(tmpdir, strChainName + '.chk');
        strPrfFile = os.path.join(tmpdir, strChainName + '.mtx');
        c.run_csprofile(strFastaFile, strResFile);
        c.chk2prf(strFastaFile, strResFile, strPrfFile);
        
    # Convert BLAST profiles to the text format OCTOPUS is using, and put them in the right folder structure
    strChainNames = os.path.join(tmpdir, 'chainnames');
    flhChainNames = file(strChainNames, 'w');
    for seq in lstSeqfiles :
        print >>flhChainNames, os.path.splitext(os.path.basename(seq))[0];
    flhChainNames.close();
    rawprfdir = os.path.join(tmpdir, 'rawprf');
    os.mkdir(rawprfdir);
    pssmdir = os.path.join(tmpdir, 'pssm');
    os.mkdir(pssmdir);
    os.system("%s %s %s %s" % (strMtx2prf, strChainNames, tmpdir, pssmdir));
    os.system("%s %s %s %s" % (strMtx2prf, strChainNames, tmpdir, rawprfdir));
    # Run OCTOPUS
    os.system("%s %s %s %s %s" % (strOctopus, strChainNames, pssmdir, rawprfdir, strOutdir));

    if debug :
        shutil.copytree(tmpdir, os.path.join(strOutdir, 'tmpdir'));
    shutil.rmtree(tmpdir);

def run_csblasttmhmm(lstSeqfiles, strOutdir, debug = False) :
    tmpdir = tempfile.mkdtemp(prefix='octopus_csblast_tmp_');
    sys.path.append(strCsblastdir);
    from csprofile_pipeline import csprofile;
    c = csprofile();
    c.strCsprofile_script = strCsprofilebin;
    # c.strMakematdir = '/path/to/makemat'; # fix

    for strFastaFile in lstSeqfiles :
        strChainName = os.path.splitext(os.path.basename(strFastaFile))[0];
        strResFile = os.path.join(tmpdir, strChainName + '.chk');
        strPrfFile = os.path.join(tmpdir, strChainName + '.mtx');
        c.run_csprofile(strFastaFile, strResFile);
        c.chk2prf(strFastaFile, strResFile, strPrfFile);
        
    strChainNames = os.path.join(tmpdir, 'chainnames');
    flhChainNames = file(strChainNames, 'w');
    for seq in lstSeqfiles :
        print >>flhChainNames, os.path.splitext(os.path.basename(seq))[0];
    flhChainNames.close();

    # Convert BLAST profiles to the text format modhmm is using
    os.system("%s %s %s %s" % (strMtx2prf, strChainNames, tmpdir, tmpdir));
    
    # Run tmhmm
    os.system("%s -m %s -s %s -f prf -L > %s" % (strModhmms, strProdivtmhmmModel, strFilenames, strXmlresultfile));
#     os.system("%s %s %s > %s" % (strSaxon, strXslt, strXmlresultfile, strTopfile));
#     os.system("%s %s %s top" % (strFastasplit, strTopfile, strOutdir));
    if debug :
        shutil.copytree(tmpdir, os.path.join(strOutdir, 'tmpdir'));
    shutil.rmtree(tmpdir);
        

if __name__ == '__main__' :
    # Check, parse $argv                                                                                                 
    strUsage = "Usage: %s <fasta files paths file> <outdir> <method>*N";
    if ( len(sys.argv) < len(strUsage.split('<')) ) :
        print >>sys.stderr, strUsage % os.path.basename(sys.argv[0]);
        sys.exit(1);
    
    strFasta = sys.argv[1];
    strOutdir = sys.argv[2];
    
    lstMethods = sys.argv[3:];
    
    flhFasta = file(strFasta);
    lst = [s.strip() for s in flhFasta];
    flhFasta.close();
    
    for method in lstMethods :
        timesfile = os.path.join(strOutdir, "%s-%s-times.txt" % (strFasta, method));
        start_time = datetime.datetime.now();

        outdir = os.path.join(strOutdir, method);
        if not os.path.isdir(outdir) :
            os.mkdir(outdir);
        # print 'run_' + method.replace('-','_') + '(lst, outdir)';
        eval('run_' + method.replace('-','_') + '(lst, outdir)');
        # eval('run_' + method.replace('-','_') + '(lst, outdir, True)');

        end_time = datetime.datetime.now();
        time_diff = end_time - start_time;
        flhTime = file(timesfile, 'w');
        print >>flhTime, float(time_diff.seconds) + time_diff.microseconds/1e6;
        flhTime.close;
        
        
